"""elements for presentation"""

import os.path as path
from .base import Tag, ContentFrame, FileNode

class Template(Tag):  
    """Template Strings""" 
    _tag = "textarea"
    _attrs = ("template",)


class MarkSection(Tag):
    """Markdown section"""
    _tag = "section"
    _attrs = ("markdown",)


class MarkdownFrame(ContentFrame):
    """A Segemnt to hold the markdown in html"""
    _parent_tag = MarkSection
    _child_tag = Template


class MarkdownFile(FileNode):
    """Segment to hold the file"""
    _ext = ".md"
    _frame = MarkdownFrame


def get_file_node(_path):
    """Get the file node based on file extension"""
    print(_path)
    name = path.basename(_path)
    if ".md" in name:
        return MarkdownFile(name, _path)
    return



"""walk the directory to find all source files"""

import os
import glob
from collections import Iterable

class SourceWalker(Iterable):
    
    def __init__(self, full_pattern):
        self.pattern = full_pattern
    
    def __iter__(self):
        pat = glob.iglob(self.pattern)
        for fp in pat:
            yield fp

def join(dirname, pattern):
    return SourceWalker(os.path.join(dirname, pattern))

class Walker(object):
    """walks and loads file types"""
    _patterns = ["*.md", "*.jinja2", "*.html"]

    def __init__(self, outpath):
        self.outpath = outpath
        self.walkers = map(lambda x: join(self.outpath, x), self._patterns)

    def __iter__(self):
        for walker in self.walkers:
            for fp in walker:
                yield fp


"""render the file accordingly"""

import os
import sys
import six
import shutil
import os.path as path
from jinja2 import Template
from .walk import Walker
from .base import Slides
from .elements import get_file_node

def convert_absolute(output_dir):
    if not path.isabs(output_dir):
        output_dir = path.join(os.getcwd(), output_dir)
    return output_dir

def copy_contents(output_dir):
    """copy the contents to output directory"""
    output_dir = convert_absolute(output_dir)
    if path.exists(output_dir):
        shutil.rmtree(output_dir)
    shutil.copytree(path.join(path.dirname(__file__), "contents"), output_dir)
    return None

def copy_template(output_dir, **kwargs):
    """copy the template file to output directory"""
    path_template = path.join(path.dirname(__file__), "templates/index.jinja2")
    output_file = path.join(convert_absolute(output_dir), "index.html")
    template = Template(open(path_template).read())
    fp = open(output_file, "w")
    fp.write(template.render(**kwargs))
    fp.close()

def get_slides(source_dir):
    """walking on the source dir to gather files"""
    source_dir = convert_absolute(source_dir)
    walker = Walker(source_dir)
    slides = Slides()
    for file in walker:
        file_node = get_file_node(file)
        slides.add(file_node.frame)
    six.print_("Done!")
    return slides

def render(title, source_dir, target_dir):
    """renders all the necessary changes to work"""
    copy_contents(target_dir)
    slides = get_slides(source_dir)
    copy_template(target_dir, data=str(slides), title=title)


"""setup.py"""
from setuptools import setup, find_packages

with open('README.md') as f:
    README = f.read()

with open('LICENSE') as f:
    LICENSE = f.read()

setup(
    name='rikiri',
    version='0.6',
    description='Make github presentations like a boss',
    long_description=README,
    author='Sourcepirate',
    author_email='plasmashadowx@gmail.com',
    url='https://github.com/sourcepirate/rikiri.git',
    license=LICENSE,
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    install_requires=[
        "jinja2",
        "yattag",
        "six"
    ],
    test_suite='tests',
    classifiers=[
          'License :: OSI Approved :: BSD License',
          'Development Status :: 5 - Production/Stable',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Environment :: Console'],
    scripts=['bin/rikiri']
)



""" gleam.py """

from yattag import SimpleDoc
from jinja2 import Template

MARKDOWN = 1
RST = 2
JINJA = 3

DEFAULT_TEMPLATE = """
<div class="reveal">
    <div class="slides">
			{{data}}
	</div>
</div>
"""

class HTMLDoc(SimpleDoc):
    """Doc with html method"""
    def html(self, *strgs):
        for strg in strgs:
            self._append(strg)

    def taghtml(self):
        return self, self.tag, self.html


def get_doc(*args, **kwargs):
    """get the base document to be filled"""
    return HTMLDoc().taghtml()

def transform_attr(prefix):
    """Transform all attr to be named with some prefix"""
    def inner_transform(*args):
        _attrs = {}
        _args = []
        for arg in args:
            if isinstance(arg, (tuple, list)):
                key, value = arg
                _prefix_key = "{}-{}".format(prefix, key)
                _attrs[_prefix_key] = value
            else:
                attr_prop = "{}-{}".format(prefix, arg)
                _args.append(attr_prop)
        return _args, _attrs
    return inner_transform

class Tag(object):
    """Base tag for all wrappers"""

    _tag = ""
    _attrs = ()
    _defaults = {}
    _attr_prefix = "data"

    def __init__(self, content, **kwargs):
        self._content = content
        self.childs = []
        self._attrs += kwargs.get("attrs", ())

    def add_child(self, other):
        self.childs.append(other)
        return self

    def __str__(self):
        doc, tag, html = get_doc(defaults=self._defaults)
        attrs_t = transform_attr(self._attr_prefix)
        args, atts = attrs_t(*self._attrs)
        with tag(self._tag, *args, **atts):
            html(self._content)
            for child in self.childs:
                html(str(child))
        return doc.getvalue()

class Slides(object):
    """Slides of presentations"""
    def __init__(self, slides=None):
        self.slides = slides or []
  
    def add(self, other):
        self.slides.append(other)

    def __str__(self):
        template = Template(DEFAULT_TEMPLATE)
        _data = map(str, self.slides)
        return template.render(data="\n".join(_data))

class ContentFrame(object):
    """Simple frame to get the content wrap"""

    _parent_tag = None
    _parent_attrs = ()
    _child_tag = None
    _child_attrs = ()

    def __init__(self, content):
        self.content = content

    def __str__(self):
        p_tag = self._parent_tag("", attrs=self._parent_attrs)
        c_tag = self._child_tag(self.content, attrs=self._child_attrs)
        p_tag.add_child(c_tag)
        return str(p_tag)

class FileNode(object):
    """File in a folder"""
    _ext = None
    _frame = None
    
    def __init__(self, name, path):
        self.name = name
        self.path = path

    @property
    def contents(self):
        fp = open(self.path, "r")
        _contents = fp.read()
        fp.close()
        return _contents

    @property
    def frame(self):
        return self._frame(self.contents)